----------------------------------------------------------------------------------
-- PacketData - the data to be sent out over the LAN
--
-- Written by hasmter@sanp.net.nz
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity PacketData is
    Port ( Addr           : in  STD_LOGIC_VECTOR (7 downto 0);
			  IPsource       : in  STD_LOGIC_VECTOR (31 downto 0);
			  IPdestination  : in  STD_LOGIC_VECTOR (31 downto 0);
			  PhyDestination : in  STD_LOGIC_VECTOR (47 downto 0);
			  PhySource      : in  STD_LOGIC_VECTOR (47 downto 0);
           Data           : out STD_LOGIC_VECTOR (7 downto 0);
			  EndOfData      : out STD_LOGIC);
end PacketData;

architecture Behavioral of PacketData is

	-- "Physical Address" - put the address of the PC you want to send to
	SIGNAL PhyDestination_1 : STD_LOGIC_VECTOR(7 downto 0);	
	SIGNAL PhyDestination_2 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL PhyDestination_3 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL PhyDestination_4 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL PhyDestination_5 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL PhyDestination_6 : STD_LOGIC_VECTOR(7 downto 0);

	-- "Physical Address" - put the address of the PC you want to send to
	SIGNAL PhySource_1 : STD_LOGIC_VECTOR(7 downto 0);	
	SIGNAL PhySource_2 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL PhySource_3 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL PhySource_4 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL PhySource_5 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL PhySource_6 : STD_LOGIC_VECTOR(7 downto 0);

	--"IP source" - put an unused IP - if unsure, see comment below after the source code
	SIGNAL IPsource_1 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL IPsource_2 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL IPsource_3 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL IPsource_4 : STD_LOGIC_VECTOR(7 downto 0);

	SIGNAL IPdestination_1 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL IPdestination_2 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL IPdestination_3 : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL IPdestination_4 : STD_LOGIC_VECTOR(7 downto 0);

	SIGNAL IPchecksum1 : STD_LOGIC_VECTOR(31 downto 0);
	SIGNAL IPchecksum2 : STD_LOGIC_VECTOR(31 downto 0);
	SIGNAL IPchecksum3 : STD_LOGIC_VECTOR(31 downto 0); 

begin
   IPsource_1 <= IPsource(31 downto 24);
   IPsource_2 <= IPsource(23 downto 16);
   IPsource_3 <= IPsource(15 downto 8);
   IPsource_4 <= IPsource( 7 downto 0);

	PhyDestination_1 <= PhyDestination(47 downto 40);
	PhyDestination_2 <= PhyDestination(39 downto 32);
	PhyDestination_3 <= PhyDestination(31 downto 24);
	PhyDestination_4 <= PhyDestination(23 downto 16);
	PhyDestination_5 <= PhyDestination(15 downto  8);
	PhyDestination_6 <= PhyDestination( 7 downto  0);

	PhySource_1 <= PhySource(47 downto 40);
	PhySource_2 <= PhySource(39 downto 32);
	PhySource_3 <= PhySource(31 downto 24);
	PhySource_4 <= PhySource(23 downto 16);
	PhySource_5 <= PhySource(15 downto  8);
	PhySource_6 <= PhySource( 7 downto  0);

   IPdestination_1 <= IPdestination(31 downto 24);
   IPdestination_2 <= IPdestination(23 downto 16);
   IPdestination_3 <= IPdestination(15 downto 8);
   IPdestination_4 <= IPdestination( 7 downto 0);

	IPchecksum1 <= x"0000C53F" 
		 + (IPsource_1 & x"00") + IPsource_2
		 + (IPsource_3 & x"00") + IPsource_4
		 + (IPdestination_1 & x"00") + IPdestination_2  
		 + (IPdestination_3 & x"00") + IPdestination_4;

	IPchecksum2 <=      (x"0000" & IPchecksum1(31 downto 16)) + (x"0000" & IPchecksum1(15 downto 0));	
	IPchecksum3 <= NOT ((x"0000" & IPchecksum2(31 downto 16)) + (x"0000" & IPchecksum2(15 downto 0)));


	process (addr)
	begin
		endOfData <= '0';
		case addr is
			--a Ethernet header
			when x"00" => data <= PhyDestination_1;
			when x"01" => data <= PhyDestination_2;
			when x"02" => data <= PhyDestination_3;
			when x"03" => data <= PhyDestination_4;
			when x"04" => data <= PhyDestination_5;
			when x"05" => data <= PhyDestination_6;	
			when x"06" => data <= PhySource_1;
			when x"07" => data <= PhySource_2;
			when x"08" => data <= PhySource_3;
			when x"09" => data <= PhySource_4;
			when x"0A" => data <= PhySource_5;
			when x"0B" => data <= PhySource_6;
			
			--a IP header
			when x"0C" => data <= x"08";
			when x"0D" => data <= x"00";
			when x"0E" => data <= x"45";
			when x"0F" => data <= x"00";
			when x"10" => data <= x"00";
			when x"11" => data <= x"2E";
			when x"12" => data <= x"00";
			when x"13" => data <= x"00";
			when x"14" => data <= x"00";
			when x"15" => data <= x"00";
			when x"16" => data <= x"80";
			when x"17" => data <= x"11";
			when x"18" => data <= IPchecksum3(15 downto 8);
			when x"19" => data <= IPchecksum3(7 downto 0);
			when x"1A" => data <= IPsource_1;
			when x"1B" => data <= IPsource_2;
			when x"1C" => data <= IPsource_3;
			when x"1D" => data <= IPsource_4;
			when x"1E" => data <= IPdestination_1;
			when x"1F" => data <= IPdestination_2;
			when x"20" => data <= IPdestination_3;
			when x"21" => data <= IPdestination_4;
			--a UDP header
			when x"22" => data <= x"04";
			when x"23" => data <= x"00";
			when x"24" => data <= x"04";
			when x"25" => data <= x"00";
			when x"26" => data <= x"00";
			when x"27" => data <= x"1A";
			when x"28" => data <= x"00";
			when x"29" => data <= x"00";
			
			--	a payload
			when x"2A" => data <= x"00"; --a put here the data that you want to send
			when x"2B" => data <= x"01"; --a put here the data that you want to send
			when x"2C" => data <= x"02"; --a put here the data that you want to send
			when x"2D" => data <= x"03"; --a put here the data that you want to send
			when x"2E" => data <= x"04"; --a put here the data that you want to send
			when x"2F" => data <= x"05"; --a put here the data that you want to send
			when x"30" => data <= x"06"; --a put here the data that you want to send
			when x"31" => data <= x"07"; --a put here the data that you want to send
			when x"32" => data <= x"08"; --a put here the data that you want to send
			when x"33" => data <= x"09"; --a put here the data that you want to send
			when x"34" => data <= x"0A"; --a put here the data that you want to send
			when x"35" => data <= x"0B"; --a put here the data that you want to send
			when x"36" => data <= x"0C"; --a put here the data that you want to send
			when x"37" => data <= x"0D"; --a put here the data that you want to send
			when x"38" => data <= x"0E"; --a put here the data that you want to send
			when x"39" => data <= x"0F"; --a put here the data that you want to send
			when x"3A" => data <= x"10"; --a put here the data that you want to send
			when x"3B" => data <= x"11"; --a put here the data that you want to send
			when others => data <= x"00";
								endOfData <= '1';
		end case;
	end process;
end Behavioral;
